#!FOOPERL

#
# Copyright (c) Xerox Corporation 1997, 1998. All rights reserved.
#
# License is granted to copy, to use, and to make and to use derivative
# works, provided that Xerox is acknowledged in all documentation
# pertaining to any such copy or derivative work. Xerox grants no other
# licenses expressed or implied. The Xerox trade name should not be used
# in any advertising without its written permission.
#
# XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
# MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
# FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
# express or implied warranty of any kind.
#
# These notices must be retained in any copies of any part of this software.
#


use Env;
use Time::Local; 

#------ the subroutines must change when the datafile structure changes---

# declare_fields, init_fields, check_match, and parse_input

# expects these fields as datafile entries/search parameters
sub declare_fields{
   #name used in CGI	#name displayed		# none/hour/week/month
    $data[0]="addr"; 	 $header[0]="CIDR addr"; $time[0]="none"; $tsize[0]=19;
    $data[1]="asnum";    $header[1]="AS Number"; $time[1]="none"; $tsize[1]=10;
    $data[2]="state"; 	 $header[2]="State";	 $time[2]="none"; $tsize[2]=6;
    $data[3]="change";   $header[3]="Change";    $time[3]="none"; $tsize[3]=19;
    $data[4]="age";      $header[4]="Appeared";  $time[4]="none"; $tsize[4]=19;
    $data[5]="metric";   $header[5]="Metric";    $time[5]="hour";
    $data[6]="flaps";    $header[6]="Flaps";     $time[6]="hour";
    $data[7]="updates";  $header[7]="Updates";   $time[7]="hour";
    $data[8]="wflaps";   $header[8]="Flaps";     $time[8]="week";
    $data[9]="whour";    $header[9]="Uptime(%)"; $time[9]="week";
    $data[10]="wcount";  $header[10]="Hours #";  $time[10]="week";
    $data[11]="wstates"; $header[11]="Changes";  $time[11]="week";
    $data[12]="mflaps";  $header[12]="Flaps";    $time[12]="month";
    $data[13]="mhour";   $header[13]="Uptime(%)"; $time[13]="month";
    $data[14]="mcount";  $header[14]="Hours #";  $time[14]="month";
    $data[15]="mstates"; $header[15]="Changes";  $time[15]="month";
    $data[16]="lflaps";  $header[16]="Flaps";    $time[16]="lweek";
    $data[17]="lhour";   $header[17]="Uptime(%)"; $time[17]="lweek";
    $data[18]="lcount";  $header[18]="Hours #";  $time[18]="lweek";
    $data[19]="lstates"; $header[19]="Changes";  $time[19]="lweek";
    $total = 20;   	# total number of fields
    $special = 5;  	# fields < this number are not max/min type fields
    # define a hash for quick dataname to data number lookup
    for($i=0; $i < $total; $i++) {
        $fndata{$data[$i]} = $i;
    }
}

#set default display, use, and search values for fields
sub init_fields {
   # default is to care about each parameter, but not display it
   for($i=0; $i < $total; $i++) {
      $care[$i] = 1;
      $use[$i] = 0;
      $max[$i] = 9999;
      $min[$i] = 0;
   }
   # special handling for these fields
   $use[0] = 1;  #always display the address
   $max[0] = "0/0"; # address doesn't really have a max/min
   $max[1] = "-5"; # default AS number, treated as don't care
   $max[2] = "up"; # state is either up or down, doesn't have max/min 
   $min[3] = "852076800"; # min starting date, Jan 1, 1997
   $max[3] = "1609372800"; # max starting date, Dec 31, 2020
   $min[4] = "852076800"; # min starting date, Jan 1, 1997
   $max[4] = "1609372800"; # max starting date, Dec 31, 2020
   $distnum = 6; # by default, distribution based on num of flaps
   $interval = 10; # by default the distribution interval is 10
   $distmax = 60; # by default, 60 is the top of the distribution
   $sortnum = 0; # by default, sort results by address
   $exactaddr = 0; # by default, don't match address exactly
   $text = "no";
}

# check if line from the data file matches the search
# return 1 if matches search, 0 otherwise
sub check_match {
   # check address matches
   if ($care[0] == 1) {
      ($mynet, $mymask) = split("/", $array[0]);
      ($matchnet, $matchmask) = split("/", $max[0]);
      if (($exactaddr == 1) && 
       (($mynet ne $matchnet) || ($mymask != $matchmask))) {
	 return(0);
      }
      if ($matchmask > 0) {  # shift no work good by 32
      if ($mymask < $matchmask) {
      # if match is too specific
	 return(0);
      }
      # shift over number of bits matching mask and compare
      $mynetnum = unpack("N",pack("C4",split(/\./o,$mynet)));
      $matchnum = unpack("N",pack("C4",split(/\./o,$matchnet)));
      $shift = 32 - $matchmask;
      $mynetnum2 = $mynetnum >> $shift;
      $matchnum2 = $matchnum >> $shift;
      if ($mynetnum2 != $matchnum2) {                 
	 return(0);
      }
      }
   }
   # check AS number matches
   if (($care[1]==1) && ($max[1] != -5) && ($max[1] != $array[1])) {
      return(0);
   }
   # check the state matches
   if (($care[2]==1) && ($array[2] ne $max[2])) {
      return (0);
   }
   # check the change time period
   if (($care[3]==1) && ($array[3] > $max[3] || $array[3] < $min[3])) {
       return(0);
   }
   # check the age time period
   if (($care[4]==1) && ($array[4] > $max[4] || $array[4] < $min[4])) {
       return(0);
   }
   # check normal fields 
   for($i=$special; $i < $total; $i++) {
      if (($care[$i]==1) && ($array[$i] > $max[$i]  || $array[$i] < $min[$i])){
	 return (0);
      }
      
   }
   return (1);
}

# parse the QUERY_STRING to set user specified parameters
sub parse_input{
   # the user input could change "use", "care", "max", and "min"
   @array = split("&",$QUERY_STRING);
   $size = @array;
   for($i = 0; $i < $size; $i++) {
      if (split("=", $array[$i]) != 2) {
         print "<p> ignoring strange input of $array[$i] <br> \n";
      } else {
	 ($myvar,$myval) = split("=", $array[$i]);
	 #myvar can be care, use, maxNAME, minNAME, or NAME
	 # OR distnum, distmax, or interval
	 # OR sortnum
	 # where NAME is one of the input fields
	 if ($myvar eq "care") {
	    $num = &field_num($myval);
	    if ($num < 0) {
               print "<p> ignoring strange input of $array[$i] <br> \n";
            } else {
	       $care[$num] = 0;
            }
	 } elsif ($myvar eq "text") {
	     $text = $myval;
	 } elsif ($myvar eq "distnum") {
            $num = &field_num($myval);
            if ($num < 0) {
               print "<p> ignoring strange input of @array[$i] <br> \n";
            } else {
               $distnum = $num;
            }
         } elsif ($myvar eq "sortnum") {
            $num = &field_num($myval);
            if ($num < 0) {
               print "<p> ignoring strange input of @array[$i] <br> \n";
            } else {
               $sortnum = $num;
            }
         } elsif ($myvar eq "distmax") {
            if ($myval < 0 || $myval > 9999) {
               print "<p> ignoring strange input of @array[$i] <br> \n";
            } else {
               $distmax = $myval;
            }
         } elsif ($myvar eq "interval") {
            if ($myval < 0 || $myval > 9999) {
               print "<p> ignoring strange input of @array[$i] <br> \n";
            } else {
               $interval = $myval;
            }
         } elsif ($myvar eq "use") {
	    $num = &field_num($myval);
	    if ($num < 0) {
               print "<p> ignoring strange input of $array[$i] <br> \n";
            } else {
	       $use[$num] = 1;
            }
         } elsif ($myvar =~ "max") { 
	    ($junk,$myfield) = split("max",$myvar);
	    $num = &field_num($myfield);
	    if ($num < 0 || $myval < 0 || $myval > 9999) {
               print "<p> ignoring strange input of $array[$i] <br> \n";
            } else {
	       if (($myfield ne "change") && ($myfield ne "age")) {
	          $max[$num] = $myval;
               } else {
		   if (split("%2F",$myval) == 3) {
		       # the user entered a date in mo/day/yr,hr:mn form
		       ($cmo,$cdy,$crest) = split("%2F",$myval);
		       ($cyr,$chtime) = split("%2C",$crest);
		       ($chr,$cmn) = split("%3A",$chtime);
		       $csc = 0;$cmo = $cmo -1;
		       if ($cyr > 1900) {
			   $cyr = $cyr - 1900;
		       }
		       $max[$num] = timegm($csc,$cmn,$chr,$cdy,$cmo,$cyr);
		   } else {
		       # the user entered a seconds value
		       $max[$num] = $myval;
		   }
               }
            }
         } elsif ($myvar =~ "min") { 
	    ($junk,$myfield) = split("min",$myvar);
	    $num = &field_num($myfield);
	    if ($num < 0 || $myval < 0 || $myval > 9999) {
               print "<p> ignoring strange input of $array[$i] <br> \n";
            } else {
	       if (($myfield ne "change") && ($myfield ne "age")) {
	          $min[$num] = $myval;
               } else {
		   if (split("%2F",$myval) == 3) {
		       # the user entered a date in mo/day/yr,hr:mn form
		       ($cmo,$cdy,$crest) = split("%2F",$myval);
		       ($cyr,$chtime) = split("%2C",$crest);
		       ($chr,$cmn) = split("%3A",$chtime);
		       $csc = 0; $cmo = $cmo -1;
		        if ($cyr > 1900) {
			   $cyr = $cyr - 1900;
		       }
		       $min[$num] = timegm($csc,$cmn,$chr,$cdy,$cmo,$cyr);
		   } else {
		       # the user entered a seconds valued
		       $min[$num] = $myval;
		   }
	       }
            }
         } else { 
	    if ($myvar eq "addr") {
	       # if adddr has an = sign, it is an exact match request
	       if (split("%3D",$myval) == 2) {
		  ($junk,$newmyval) = split("%3D",$myval);
		  $exactaddr = 1;
		  $myval = $newmyval;
               }
	       ($newnet,$newmask) = split("%2F",$myval);
	       $max[0] = "$newnet/$newmask";
            } elsif ($myvar eq "asnum") {
	       $num = &field_num("asnum");
	       if ($num < -1) {
                  print "<p> ignoring strange input of $array[$i] <br> \n";
               } else {
	          $max[$num] = $myval;
               }
            } elsif ($myvar eq "state") {
	       $num = &field_num("state");
	       if ($num < 0) {
                  print "<p> ignoring strange input of $array[$i] <br> \n";
               } else {
	          $max[$num] = $myval;
               }
            } else {
               print "<p> HUH ignoring strange input of $array[$i] <br> \n";
            }

         }
      }
   }
}

# find a number to match the field name
sub field_num{
   my($name) = @_[0];
   if (defined($fndata{$name})) {
       return ($fndata{$name});
   } else {
       return (-1);
   }
}


#open the data file and check its valid
sub file_open {
   open(DATAFILE, $datafile);
   # verify the correct first line of the file
   $line = <DATAFILE>;
   ($Route,$Data,$From, $utime, $fdate, $totalw, $totalm) = split(" ",$line);
   if ($Route ne "Route") {
      print "Error: problem reading from datafile ($datafile) <br>\n";
      print "starting line was: $line <br> \n";
      print "Please report this problem to the webmaster at this site.\n";
      print "Often this error means the site has not set the proper \n";
      print "permissions for the datafile or has chosen not to make the\n";
      print "datafile searchable from the web.\n";
      print "</body> </html> \n";
      exit;
   }
}

sub sortfield {
   @linea = split(" ", $a);
   @lineb = split(" ", $b);
   if ($sortnum < $special) {
      # sort by address
      if ($sortnum == 0) {
         ($subneta,$maska) = (split(m|/|o, $linea[0]));
	 ($subnetb,$maskb) = (split(m|/|o, $lineb[0]));
	 $netnuma = unpack("N",pack("C4",split(/\./o,$subneta)));
	 $netnumb = unpack("N",pack("C4",split(/\./o,$subnetb)));
         if ($netnuma == $netnumb) {
            return ($maska <=> $maskb);
         } else {
            return ($netnuma <=> $netnumb);
         }
      } elsif ($sortnum == 1) {
         if ($linea[$sortnum] > $lineb[$sortnum]) {
	    return (-1);
         } elsif ($linea[$sortnum] < $lineb[$sortnum]) {
	    return (1);
         } else {
	    return (0);
         }
      } elsif ($sortnum == 2) {
	 if ($linea[2] eq "up") {
	    return (-1);
         } elsif ($lineb[2] eq "up") {
	    return (1);
         } else {
	    return (-1);
         }
      } elsif ($sortnum == 3) {
	  if ($linea[$sortnum] > $lineb[$sortnum]) {
	    return (-1);
         } elsif ($linea[$sortnum] < $lineb[$sortnum]) {
	    return (1);
         } else {
	    return (0);
         }
      } elsif ($sortnum == 4) {
	  if ($linea[$sortnum] > $lineb[$sortnum]) {
	    return (-1);
         } elsif ($linea[$sortnum] < $lineb[$sortnum]) {
	    return (1);
         } else {
	    return (0);
         }
      } else {
	 return (-1);
      }
   } elsif ($sortnum < $total) {
      if ($linea[$sortnum] > $lineb[$sortnum]) {
	 return (-1);
      } elsif ($linea[$sortnum] < $lineb[$sortnum]) {
	 return (1);
      } else {
	 return (0);
      }
   } else {
      return (0);
   }
}

1;
