#
# Copyright (c) Xerox Corporation 1997, 1998. All rights reserved.
#
# License is granted to copy, to use, and to make and to use derivative
# works, provided that Xerox is acknowledged in all documentation
# pertaining to any such copy or derivative work. Xerox grants no other
# licenses expressed or implied. The Xerox trade name should not be used
# in any advertising without its written permission.
#
# XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
# MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
# FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
# express or implied warranty of any kind.
#
# These notices must be retained in any copies of any part of this software.
#


open(LOG,">./install.log") ||
    die "Unable to create or open ./install.log.  check write permissions??\n";

#------ in case user enters ./mydir, get pwd to replace the .
$pwd = `pwd`;
chop($pwd);
$pwd = "$pwd/";
#------- in case user enters ~/mydir, get home to replace the ~
($jn,$jp,$ju,$jg,$jq,$jc,$js,$home,$jh) = getpwnam($ENV{"USER"});


#------ start the install
&print_intro;
&verify_perl;
&build_routewatch;
&get_bindir;
&get_outdir;
&get_arcdir;
$webserver = &have_web;
if ($webserver eq "no") {
    &no_web;
}
&get_webdir;
&get_topology;
$cgiserver = &have_cgi;
if ($cgiserver eq "no") {
    &no_cgi;
}
&get_cgidir;
&get_url;


print "\n\n\n\n";
print "The RouteMonitor Package is now being installed....\n";
$myrw = "$mybin/routewatch";
$myrm = "$mybin/routemonitor";
$myas = "$mybin/aslookup";
$mysearch = "$myhref/rmsearch.cgi";
$mydist = "$myhref/rmdist.cgi";
print "Installing executables into $mybin.... \n";
&copy_file("./routemonitor/routewatch", "$mybin/routewatch");
&build_perl("./routemonitor/dump_table", "$mybin/dump_table");
&build_perl("./routemonitor/reset_table", "$mybin/reset_table");
&build_perl("./routemonitor/routemonitor", "$mybin/routemonitor");
&build_perl("./htmlmaker/htmlmaker", "$mybin/htmlmaker");
&build_perl("./htmlmaker/aslookup", "$mybin/aslookup");
&build_perl("./htmlmaker/asretry", "$mybin/asretry");
&build_perl("./htmlmaker/statsedit", "$mybin/statsedit");


print "Installing Html and web database into $mywdir.... \n";
&build_perl("./html/about.html","$mywdir/about.html");
&build_perl("./html/dcustom.html","$mywdir/dcustom.html");
&build_perl("./html/defs.html","$mywdir/defs.html");
&build_perl("./html/dflaps.html","$mywdir/dflaps.html");
&build_perl("./html/index.html","$mywdir/index.html");
&build_perl("./html/last.html","$mywdir/last.html");
&build_perl("./html/menu.html","$mywdir/menu.html");
&build_perl("./html/month.html","$mywdir/month.html");
&build_perl("./html/saddress.html","$mywdir/saddress.html");
&build_perl("./html/sasnum.html","$mywdir/sasnum.html");
&build_perl("./html/scustom.html","$mywdir/scustom.html");
&build_perl("./html/smenu.html","$mywdir/smenu.html");
&build_perl("./html/summary.html","$mywdir/summary.html");
&build_perl("./html/week.html","$mywdir/week.html");
&copy_file("./html/route_stats","$mywdir/route_stats");
&copy_file("./html/stone.gif", "$mywdir/stone.gif");
&copy_file("./html/hline.gif", "$mywdir/hline.gif");

print "Installing CGI scripts in $mycdir.... \n";
&build_perl("./cgi/rmsearch.cgi", "$mycdir/rmsearch.cgi");
&build_perl("./cgi/rmdist.cgi", "$mycdir/rmdist.cgi");
&build_perl("./cgi/Fields.pl","$mycdir/Fields.pl");
&build_perl("./cgi/Html.pl","$mycdir/Html.pl");

print "Main installation completed.  Configuring options....\n";
&archives;
&use_config;
&start_stop;
&set_permissions;
&install_man;
&messages;
&register;
close(LOG);
exit;


#==================================================================
sub print_intro {
    system("clear");
    print "You are about to install the Multicast RouteMonitor package.\n";
    print "The Multicast RouteMonitor package will collect data on\n";
    print "the Mbone topology, as seen from your site.  The resulting data\n";
    print "will be displayed on a local web page.  Data on the web page\n";
    print "will be updated hourly.\n";
    print "\n";
    print "This script should handle the install process by asking you\n";
    print "several questions.  Default answers to the questions are shown\n";
    print "between brackets like: [default]\n";
    print "You may simply hit return to use the default answer.\n";
    $mydate = `date`;
    print LOG "Install started at $mydate\n";
}


sub verify_perl {
    print "\n\n\n\n";
    if (defined($ARGV[0])) {
	$myperl = $ARGV[0];
    } else {
	$myperl = "/foo/dir/perl";
	foreach $dir (split(":", $ENV{"PATH"})) { 
	    if ( -x "$dir/perl") {
		$myperl = "$dir/perl";
		last;
	    } 
	}
    }
    $ans = "/dan/foo/bar";
    while(! -x $ans) {
	print "Many RouteMonitor tools are perl scripts.\n";
	print "Perl v5.0 or higher is required\n";
	print "Enter the location of Perl:[$myperl] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $myperl;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;
	if (! -x $ans) {
	    print "The file $ans is not executable.\n";
	    print "Provide the complete path for perl on your system.\n";
	    print "You may be able to obtain this from: which perl\n\n";
	}
    }
    $myperl = $ans;
    print LOG "perl location is: $ans\n";
}

sub build_routewatch {
    print "\n\n\n\n";
    if (-x "./routemonitor/routewatch") {
	print "A binary version of routewatch exits already.\n";
	print "Use this binary file: [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    print LOG "Using existing routewatch binary\n";
	    return;
	}
    }
    print "Attempting to compile ./routewatch/routewatch.c\n";
    $myc = gcc;
    $ans = "/dan/foo/bar";
    while(! -x $ans) {
	print "The C compiler is: [$myc] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $myc;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;
	foreach $dir (split(":", $ENV{"PATH"})) { 
	    if ( -x "$dir/$ans") {
		$ans = "$dir/$ans";
		last;
	    } 
	}
	if (! -x $ans) {
	    print "The file $ans is not executable.\n";
	    print "Try giving the complete path for the compiler.\n";
	    print "\n\n";
	}
    }
    $myc = $ans;
    $myl = "-lsocket -lnsl";
    $ans = "/dan/foo/bar";
    print "Libraries, command line options, etc:\n";
    print "ex: Solaris requires:  -lsocket -lnsl\n";
    print "ex: SunOS requires no libraries:  none \n";
    print "--  type none to indicate no libraries or options\n";
    print "Libraries and options: [$myl] ";
    $ans = <STDIN>;
    if ($ans eq "\n") {
	$ans = $myl;
    } else {	
	chop($ans);
	if ($ans =~ "none") {
	    $ans = " ";
	}
    }
    $myl = $ans;
    &build_makefile;
    if (! -x "./routemonitor/routewatch") {
	print "Unable to build the routewatch... \n";
	print "The routewatch.c program and a makefile are located in:\n";
	print "./routemonitor\n";
	print "You will need to build the executable routewatch.  Please\n";
	print "do this and then run the installation program again.\n";
	print "If you are unable to build the routewatch program, send\n";
	print 'mail to masseyd@cs.ucla.edu for technical support.';
	print "\n";
	print LOG "Unable to build ./routemonitor/routewatch\n";
	close(LOG);
	exit;
    }
    print LOG "Built new ./routemonitor/routewatch\n";
}

sub build_makefile {
    open(INFILE, "./routemonitor/makefile.config") || 
	die "Can't find makefile ./routemonitor.makefile.config\n";
    open(OUTFILE, ">./routemonitor/makefile") || 
	die "Can't create makefile ./routemonitor/makefile\n";
    while($line = <INFILE>) {
	$line =~ s#FOOCC#$myc#g;
	$line =~ s#FOOLIBS#$myl#g;
	print OUTFILE "$line";
    }
    close(INFILE);
    close(OUTFILE);
    $ret = system('cd ./routemonitor; make');
    if ($ret != 0) {
	print "Failed to make routewatch\n";
    }
}

sub get_bindir {
    $mybin = "/usr/local/bin";
    $ans = "/dan/foo/bar";
    print "\n\n\n\n";
    print "The RouteMonitor Package includes the following executables:\n";
    print  "htmlmaker, routemonitor, routewatch, dump_table\n";
    print "reset_table, statsedit, asretry, and aslookup\n";
    while(! -d $ans) {
	print "Select a directory for the RouteMonitor executables: [$mybin] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $mybin;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;
	$ans =~ s#/$##g;
	if (! -e $ans) {
	    print "$ans does not exist\n";
	    print "create $ans: [yes] ";
	    $yes = <STDIN>;
	    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		$ret = system("mkdir -p $ans");
		if ($ret != 0) {
		    print "Unable to create directory $ans\n";
		}
	    }
	}
	if (! -d $ans) {
	    print "$ans is not a directory. \n";
	    
	}
    }
    $mybin = $ans;
    print LOG "executable directory is: $mybin\n";
}

sub get_outdir {
    $myrdir = "/usr/local/etc/routemonitor";
    $ans = "/dan/foo/bar";
    print "\n\n\n\n";
    print "RouteMonitor executables require a working directory to\n";
    print "write error logs, process id files, route table dumps, and\n";
    print "misc files.   If you use a routemonitor config file, the\n";
    print "config file must also go in this directory.\n";
    while(! -d $ans) {
	print "Enter the RouteMonitor working directory: [$myrdir] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $myrdir;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;
	$ans =~ s#/$##g;
	if (! -e $ans) {
	    print "$ans does not exist\n";
	    print "create $ans: [yes] ";
	    $yes = <STDIN>;
	    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		$ret = system("mkdir -p $ans");
		if ($ret != 0) {
		    print "Unable to create directory $ans\n";
		}
	    }
	}
	if (! -d $ans) {
	    print "$ans is not a directory. \n";
	}
    }
    $myrdir = $ans;
    $myhdir = $ans;
    print LOG "working directory is: $myrdir\n";
}

sub get_arcdir {
    $myadir = "$myhdir/archives";
    $ans = "/dan/foo/bar";
    print "\n\n\n\n";
    print "If htmlmaker is run with the -T, -W, or -M option, htmlmaker\n";
    print "will archive copies of the hourly route_table, weekly route\n";
    print "statistics, or monthly route statistics (respectively).\n";
     while(! -d $ans) {
	 print "Enter the directory for possible archive output: [$myadir] ";
	 $ans = <STDIN>;
	 if ($ans eq "\n") {
	     $ans = $myadir;
	 } else {
	     chop($ans);
	 }
	 $ans =~ s#\./#$pwd#g;
	 $ans =~ s#~/#$home/#g;
	 $ans =~ s#/$##g;
	 if (! -e $ans) {
	     print "$ans does not exist\n";
	     print "create $ans: [yes] ";
	     $yes = <STDIN>;
	     if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		 $ret = system("mkdir -p $ans");
		 if ($ret != 0) {
		     print "Unable to create directory $ans\n";
		 }
	     }
	 }
	 if (! -d $ans) {
	     print "$ans is not a directory. \n";
	 }
     }
    $myadir = $ans;
    print LOG "archive directory is: $myadir\n";
}

sub have_web {
    print "\n\n\n\n";
    print "RouteMonitor generates hourly web pages.  Ideally RouteMonitor\n";
    print "can simply write into a directory served by your web server. \n";
    print "Can RouteMonitor write directly to a web server directory: [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	print LOG "RouteMonitor can write to web server directory\n";
	return("yes");
    } else {
	print LOG "RouteMonitor can not write to web server directory\n";
	return("no");
    }
}

sub no_web {
    print "RouteMonitor will write the hourly html files and route stats\n";
    print "database to a local directory.  You must supply a shell script\n";
    print "(or other program) which copies the contents of this directory\n";
    print "to a directory on the web server.  When RouteMonitor produces a\n";
    print "set of new files for the web, it will automatically invoke your\n";
    print "script.\n";
    print "\n";
    print "A typical script uses tools such as rcp, ssh, or ftp.  For now,\n";
    print "RouteMonitor needs to know the name and location of the script.\n";
    print "\n";
    print "Enter the script name: [$mybin/webcopy] ";
    $ans = <STDIN>;
    if ($ans eq "\n") {
	$ans = "$mybin/webcopy";
    } else {
	chop($ans);
    }
    $ans =~ s#\./#$pwd#g;
    $ans =~ s#~/#$home/#g;
    $mycopy = $ans;
    print LOG "web copy script is: $mycopy\n";
    print "\n";
    print "Select the local directory where RouteMonitor put html files.\n";
    print "Remember the script $mycopy must copy all files \n";
    print "from this directory to a directory on the web server.\n";
}
sub get_webdir {
    $mywdir = "/foo/webserver/RouteMonitor";
    $ans = "/dan/foo/bar";
    print "\n\n\n\n";
    while(! -d $ans) {
	print "Enter the directory for html output: [$mywdir] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $mywdir;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;
	$ans =~ s#/$##g;
	if (! -e $ans) {
	    print "$ans does not exist\n";
	    print "create $ans: [yes] ";
	    $yes = <STDIN>;
	    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		$ret = system("mkdir -p $ans");
		if ($ret != 0) {
		    print "Unable to create directory $ans\n";
		}
	    }
	}
	if (! -d $ans) {
	    print "$ans is not a directory. \n";
	}
    }
    $mywdir = $ans;
    print LOG "web server directory is: $mywdir\n";
}

sub have_cgi {
    print "\n\n\n\n";
    if ($webserver eq "no") {
	return("no");
    }
    print "RouteMonitor packages includes some CGI scripts that should\n ";
    print "be installed in a cgi directory on the web server.\n";
    print "Can the install program access a CGI directory: [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	print LOG "install can write to a cgi directory\n";
	return("yes");
    } else {
	print LOG "install can not write to cgi directory\n";
	return("no");
    }
}

sub no_cgi {
    print "RouteMonitor will install cgi scripts into a local directory\n";
    print "After the install is complete, you will need to copy all of\n";
    print "the files in the local cgi directory onto a a cgi directory\n";
    print "on the web server.\n";
    print "Select a local directory in which to install cgi scripts.\n";
}

sub get_cgidir {
    $mycdir = "/foo/webserver/cgi/RouteMonitor";
    $ans = "/dan/foo/bar";
    while(! -d $ans) {
	print "Enter the directory for cgi scripts: [$mycdir] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $mycdir;
	} else {
	    chop($ans);
	}
	$ans =~ s#\./#$pwd#g;
	$ans =~ s#~/#$home/#g;	
	$ans =~ s#/$##g;
	if (! -e $ans) {
	    print "$ans does not exist\n";
	    print "create $ans: [yes] ";
	    $yes = <STDIN>;
	    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		$ret = system("mkdir -p $ans");
		if ($ret != 0) {
		    print "Unable to create directory $ans\n";
		}
	    }
	}
	if (! -d $ans) {
	    print "$ans is not a directory. \n";
	}
    }
    $mycdir = $ans;
    print LOG "cgi directory is: $mycdir\n";
}

sub get_url {
    $myhref = "http://www.mysite.edu/cgi/";
    $ans = "/dan/foo/bar";
    print "\n\n\n\n";	
    print "Some files produced by RouteMonitor will include hyper text \n";
    print "links to the cgi scripts.  You must provide the start of a URL\n";
    print "and RouteMonitor will append the cgi script name (ie rmsearch.cgi) \n";
    print "to the end of this URL\n";
    while($ans !~ "http://") {
	print "initial url: [$myhref] ";
	$ans = <STDIN>;
	if ($ans eq "\n") {
	    $ans = $myhref;
	} else {
	    chop($ans);
	    $ans =~ s#/$##g;
	}
	if ($ans !~ "http://") {
	    print "$ans is not a valid beginning for cgi url. \n";
	}
    }
    $myhref = $ans;
    $myhref =~ s#/$##g;
    print LOG "cgi script url starts with $myhref\n";
}


sub build_perl {
    $in = $_[0];
    $out = $_[1];
    if ($in eq $out) {
	print "installing $in into directory $out ??\n";
	print "You can not use the src code dir as the executable dir.\n";
	print "if you really want to install here, how about...\n";
	print "using ./bin as the RouteMonitor executable directory.\n";
	print "Sorry, you will need to run the install script again\n";
	print LOG "source directory was same as install directory!\n";
	close(LOG);
	exit;
    }
    if (-e $out) {
	print "$out already exists.\n";
	print "Overwrite existing $out: [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    print "replacing $out\n";
	} else {
	    print LOG "new $out not installed, using existing file instead\n";
	    print "using existing $out\n";
	    return(1);
	}
    }
    open(INFILE, "$in") || 
	die "Can't open $in \n";
    open(OUTFILE, ">$out") || 
	die "Can't create $out\n";
    while($line = <INFILE>) {
	$line =~ s#FOOPERL#$myperl#g;
	$line =~ s#FOORTEMONDIR#$myrdir#g;
	$line =~ s#FOOHMKRDIR#$myhdir#g;
	$line =~ s#FOOARDIR#$myadir#g;
	$line =~ s#FOOWEBDIR#$mywdir#g;
	$line =~ s#FOORTEEXEC#$myrm#g;
	$line =~ s#FOORWEXEC#$myrw#g;
	$line =~ s#FOOCGISEARCH#$mysearch#g;
	$line =~ s#FOOCGIDIST#$mydist#g;
	$line =~ s#FOOASLEXEC#$myas#g;
	if ($line =~ "FOOTOPOLOGY") {
	   $tsize = @mytopo;
	   for($count = 0; $count < $tsize; $count++) {
		print OUTFILE "$mytopo[$count]";
	   }
	} else {
	   print OUTFILE "$line";
        }
    }
    close(INFILE);
    close(OUTFILE);
}

sub copy_file {
    $in = $_[0];
    $out = $_[1];
    if ($in eq $out) {
	print "installing $in into directory $out ??\n";
	print "You can not use the src code dir as the executable dir.\n";
	print "if you really want to install here, how about...\n";
	print "using ./bin as the RouteMonitor executable directory.\n";
	print "Sorry, you will need to run the install script again\n";
	print LOG "source directory was same as install directory!\n";
	close(LOG);
	exit;
    }
    if (-e $out) {
	print "$out already exists. \n";
	print "Overwrite existing $out: [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    print "replacing $out\n";
	} else {
	    print "using existing $out\n";
	    print LOG "new $out not installed, using existing file instead\n";
	    return(1);
	}
    }
    $ret = system("cp $in $out");
    if ($ret != 0) {
	print "Unable to copy $in to $out\n";
	print LOG "copy failed for: cp $in $out\n";
	return(-1);
    }
}

sub use_config {
    &build_perl("./routemonitor/route_monitor.conf","$myrdir/route_monitor.conf");
    print LOG "route_monitor.conf installed in $myrdir/route_monitor.conf\n";
    print "\n\n";
    print "By default, RouteMonitor listens to updates from all neighboring\n";
    print "multicast routers.  The distance to each neighbor is assumed to\n";
    print "1.  If you would like to assign distance metrics to neighboring\n";
    print "routers and/or subnets, edit the file:\n";
    print "$myrdir/route_monitor.conf\n";
    print "The file contains instructions on how to assign metrics.\n\n";
}

sub archives {
    print "\n\n";
    print "RouteMonitor can archive hourly route tables, weekly route\n";
    print "statistics, monthly statistics, or any combination of the three.\n";
    print "Would you like to archive hourly route tables [no] ";
    $yes = <STDIN>;
    if (($yes =~ "y") || ($yes =~ "Y")) {
	print "route table archives will be written to:\n";
	print "$myadir/route_table.year.yearday.hour\n";
	print LOG "Route Tables archived to $myadir/route_table.yr.yrday.hr\n";
	$harchive = "yes";
    } else {
	$harchive = "no";
    }
    print "Would you like to archive weekly route statistics [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	print "weekly route statistic archives will be written to:\n";
	print "$myadir/route_stats.year.weeknum\n";
	print LOG "Week route stats archived to $myadir/route_stats.yr.wknm\n";
	$warchive = "yes";
    } else {
	$warchive = "no";
    }
    print "Would you like to archive monthly route statistics [no] ";
    $yes = <STDIN>;
    if (($yes =~ "y") || ($yes =~ "Y")) {	
	print "monthly route statistic archives will be written to:\n";
	print "$myadir/route_stats.year.month\n";
	print LOG "Monthly route stats archived to $myadir/route_stats.yr.mo.\n";
	$marchive = "yes";
    } else {
	$marchive = "no";
    }
}

sub start_stop {
    $opts = " ";
    if ($marchive eq "yes") {
	$opts = "$opts -M";
    }
    if ($warchive eq "yes") {
	$opts = "$opts -W";
    }
    if ($harchive eq "yes") {
	$opts = "$opts -T";
    }
    if ($webserver eq "no") {
	$opts = "-R $mycopy";
    }
    print "Creating start/stop script for RouteMonitor...\n";
    open(START,">$mybin/RouteMonitor") ||
	die "Unable to create $mybin/RouteMonitor\n";
    print START '#! /bin/sh'; print START "\n";
    print START '#'; print START "\n";
    print START '#ident  "@(#)RouteMonitor 1.0    "'; print START "\n";
    print START '#'; print START "\n";
    print START '# start/stop the multicast route monitor'; print START "\n";
    print START "\n";
    print START 'case "$1" in'; print START "\n";
    print START "\n";
    print START '\'start\')'; print START "\n";
    print START '        # Start the multicast route monitor';print START "\n";
    print START '        if [ -f '; print START "$mybin/htmlmaker"; print START ' ] ; then'; print START "\n";  
    print START '            echo "starting multicast route monitor"'; print START "\n";
    print START "            $mybin/htmlmaker $opts &\n";
    print START '        fi'; print START "\n";
    print START '        ;;'; print START "\n";
    print START "\n";
    print START '\'stop\')'; print START "\n";
    print START '        # Stop the multicast route monitor'; print START "\n";
    print START '        PID=`awk \'{print $1}\' '; print START "$myrdir/html_maker.pid"; print START '`'; print START "\n";
    print START '        if [ ! -z "$PID" ] ;  then'; print START "\n";
    print START '            kill ${PID} 1> /dev/null 2>&1 || echo "htmlmaker pid = ${PID} not killed"'; print START "\n";
    print START '        fi'; print START "\n";
    print START '        PID=`awk \'{print $1}\' '; print START "$myrdir/as_lookup.pid"; print START '`'; print START "\n";
    print START '        if [ ! -z "$PID" ] ;  then'; print START "\n";
    print START '            kill ${PID} 1> /dev/null 2>&1'; print START "\n";
    print START '        fi'; print START "\n";
    print START '        PID=`awk \'{print $1}\' '; print START "$myrdir/route_monitor.pid"; print START '`'; print START "\n";
    print START '        if [ ! -z "$PID" ] ;  then'; print START "\n";
    print START '            kill ${PID} 1> /dev/null 2>&1 || echo "routemonitor pid = ${PID} not killed"'; print START "\n";
    print START '        fi'; print START "\n";
    print START '        ;;'; print START "\n";
    print START '\'clean\')'; print START "\n";
    print START '        # remove the logs'; print START "\n";
    print START "        rm $myrdir/route_monitor.err\n";
    print START "        rm $myrdir/html_maker.err\n";
    print START "        rm $myrdir/as_lookup.err\n";
    print START "        rm $myrdir/find_as_for\n";
    print START "        rm $myrdir/found_as_for\n";
    print START '        ;;'; print START "\n";
    print START '*)'; print START "\n";
    print START '        # if error, lets just start';print START "\n";
    print START '        $0 start'; print START "\n";
    print START '        ;;'; print START "\n";
    print START "\n";
    print START 'esac'; print START "\n";
    print START 'exit 0'; print START "\n";
}


sub install_man {
    print "\n\n";
    $man = "yes";
    $fail = "no";
    while ($man eq "yes") {
	print "Install man pages? [yes] ";
    	$yes = <STDIN>;
    	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    print "Directory for man pages: [/usr/local/man] ";
	    $ans = <STDIN>;
	    if ($ans eq "\n") {
		$ans = "/usr/local/man";
    	    } else {
		chop($ans);
	    }
	    $ans =~ s#\./#$pwd#g;
	    $ans =~ s#~/#$home/#g;
	    if ((-d "$ans/man1") && (-d "$ans/man8")) {
		&copy_file("./man/aslookup.8","$ans/man8/aslookup.8");
		$ret = system("chmod a+r $ans/man8/aslookup.8");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
		"Unable to set file permissions for $ans/man8/aslookup.8...\n";
		}
		&copy_file("./man/asretry.8","$ans/man8/asretry.8");
		$ret = system("chmod a+r $ans/man8/asretry.8");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
		"Unable to set file permissions for $ans/man8/asretry.8...\n";
		}
		&copy_file("./man/statsedit.8","$ans/man8/statsedit.8");
		$ret = system("chmod a+r $ans/man8/statsedit.8");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
		"Unable to set file permissions for $ans/man8/statsedit.8...\n";
		}
		&copy_file("./man/routewatch.8","$ans/man8/routewatch.8");
		$ret = system("chmod a+r $ans/man8/routewatch.8");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
	     "Unable to set file permissions for $ans/man8/routewatch.8...\n";
		}
		&copy_file("./man/dump_table.1","$ans/man1/dump_table.1");
		$ret = system("chmod a+r $ans/man1/dump_table.1");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
	     "Unable to set file permissions for $ans/man1/dump_table.1...\n";
		}
		&copy_file("./man/reset_table.1","$ans/man1/reset_table.1");
		$ret = system("chmod a+r $ans/man1/reset_table.1");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
	     "Unable to set file permissions for $ans/man1/reset_table.1...\n";
		}
		&copy_file("./man/routemonitor.1","$ans/man1/routemonitor.1");
		$ret = system("chmod a+r $ans/man1/routemonitor.1");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
	    "Unable to set file permissions for $ans/man1/routemonitor.1...\n";
		}
		&copy_file("./man/htmlmaker.1","$ans/man1/htmlmaker.1");
		$ret = system("chmod a+r $ans/man1/htmlmaker.1");
		if ($ret != 0) {
		    $fail = "yes";
		    print 
	     "Unable to set file permissions for $ans/man1/htmlmaker.1...\n";
		}
		print "Run /usr/bin/catman -w -M $ans ? [yes] ";
		$yes = <STDIN>;
    		if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
		    $ret = system("/usr/bin/catman -w -M $ans");
		    if ($ret != 0) {
			print "/usr/bin/catman -w -M $ans failed....\n";
		    }
		}
		$man = "no";
		if ($fail eq "yes") {
		    print LOG "error while installing man pages...\n";
		} else {
		    print LOG "man pages installed.\n";
		}
	    } else {
		print "Directories $ans/man1 and $ans/man8 do not exist??\n";
 	    }
        } else {
	    print LOG "man pages not installed\n";
	    $man = "no";
 	}
    }
}

sub set_permissions {
    # html files and directory
    $user = "foo";
    $group = "foo";
    print "\n\n\n\n";
    print "Setting the correct owner/group/permissions for files...\n";
    print "Set owner/group/permissions for web accessible files [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	$fail = "no";
	print "File owner: [$user] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $user = $ans;
	}
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	print "Permissions: [664]  ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $perm = $ans;
	} else {
	    $perm = "664";
	}
	foreach $nfile ("about.html", "dcustom.html", "defs.html", 
		  "dflaps.html", "last.html", 
		  "hline.gif", "index.html", "menu.html", "month.html", 
		  "saddress.html", "sasnum.html", "scustom.html", 
		  "smenu.html","stone.gif", "summary.html", "week.html", 
		  "route_stats" ) {
	    $hfile = "$mywdir/$nfile";
	    $ret = system("chown $user $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $hfile failed!\n";
	    }
	    $ret = system("chgrp $group $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $hfile failed!\n";
	    }
	    $ret = system("chmod $perm $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $hfile failed!\n";
	    }
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for html files\n";
	} else {
	    print LOG "chown/chgrp/chmod failed for html files \n";
	}
	print "\nSet owner/group/permissions for $mywdir [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    $fail = "no";
	    print "File owner: [$user] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$user = $ans;
	    }
	    print "File group: [$group] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$group = $ans;
	    }	
	    print "Permissions: [775] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$perm = $ans;
	    } else {
		$perm = "775";
	    }
	    $ret = system("chown $user $mywdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $mywdir failed!\n";
	    }
	    $ret = system("chgrp $group $mywdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $mywdir failed!\n";
	    }
	    $ret = system("chmod $perm $mywdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $mywdir failed!\n";
	    }
	    if ($fail eq "no") {
		print LOG "chown/chgrp/chmod completed for $mywdir \n";
	    } else {
		print LOG "chown/chgrp/chmod failed for $mywdir \n";
	    }
	} else {
	    print LOG "didn't chown/chgrp/chmod $mywdir\n";
	}
    } else {
	print LOG "didn't chown/chgrp/chmod html files or $mywdir\n";
    }
    print "\nSet owner/group/permissions for executable files [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	$fail = "no";
	print "File owner: [$user] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $user = $ans;
	}
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	print "Permissions: [755] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $perm = $ans;
	} else {
	    $perm = "755";
	}
	foreach $nfile ("asretry", "statsedit", "aslookup", "htmlmaker", "routemonitor", "dump_table", "reset_table", "RouteMonitor") {
	    $hfile = "$mybin/$nfile";
	    $ret = system("chown $user $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $hfile failed!\n";
	    }
	    $ret = system("chgrp $group $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $hfile failed!\n";
	    }
	    $ret = system("chmod $perm $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $hfile failed!\n";
	    }
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for executable files\n";
	} else {
	    print LOG "chown/chgrp/chmod failed for executable files \n";
	}
	print "\nSet owner/group/permissions for $mybin [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    $fail = "no";
	    print "File owner: [$user] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$user = $ans;
	    }
	    print "File group: [$group] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$group = $ans;
	    }	
	    print "Permissions: [755] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$perm = $ans;
	    } else {
		$perm = "755";
	    }
	    $ret = system("chown $user $mybin");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $mybin failed!\n";
	    }
	    $ret = system("chgrp $group $mybin");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $mybin failed!\n";
	    }
	    $ret = system("chmod $perm $mybin");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $mybin failed!\n";
	    }
	    if ($fail eq "no") {
		print LOG "chown/chgrp/chmod completed for $mybin \n";
	    } else {
		print LOG "chown/chgrp/chmod failed for $mybin \n";
	    }
	} else {
	    print LOG "didn't chown/chgrp/chmod $mybin\n";
	}
    } else {
	foreach $nfile ("asretry", "statsedit", "aslookup", "htmlmaker", "routemonitor", "dump_table", "reset_table", "RouteMonitor") {
	    $hfile = "$mybin/$nfile";
	    $ret = system("chmod +x $hfile");
	}
	print LOG "didn't chown/chgrp/chmod executable files or $mybin\n";
    }
    print "\nSet owner/group/permissions for cgi scripts [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	$fail = "no";
	print "File owner: [$user] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $user = $ans;
	}
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	print "Permissions: [755] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $perm = $ans;
	} else {
	    $perm = "755";
	}
	foreach $nfile ("Fields.pl", "Html.pl", "rmsearch.cgi", "rmdist.cgi") {
	    $hfile = "$mycdir/$nfile";
	    $ret = system("chown $user $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $hfile failed!\n";
	    }
	    $ret = system("chgrp $group $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $hfile failed!\n";
	    }
	    $ret = system("chmod $perm $hfile");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $hfile failed!\n";
	    }
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for cgi scripts\n";
	} else {
	    print LOG "chown/chgrp/chmod failed for cgi scripts \n";
	}
	print "\nSet owner/group/permissions for $mycdir [yes] ";
	$yes = <STDIN>;
	if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	    $fail = "no";
	    print "File owner: [$user] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$user = $ans;
	    }
	    print "File group: [$group] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$group = $ans;
	    }	
	    print "Permissions: [755] ";
	    $ans = <STDIN>;
	    if ($ans ne "\n") {
		chop($ans);
		$perm = $ans;
	    } else {
		$perm = "755";
	    }
	    $ret = system("chown $user $mycdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chown $user $mybin failed!\n";
	    }
	    $ret = system("chgrp $group $mycdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chgrp $group $mybin failed!\n";
	    }
	    $ret = system("chmod $perm $mycdir");
	    if ($ret != 0) {
		$fail = "yes";
		print "chmod $perm $mycdir failed!\n";
	    }
	    if ($fail eq "no") {
		print LOG "chown/chgrp/chmod completed for $mycdir \n";
	    } else {
		print LOG "chown/chgrp/chmod failed for $mycdir \n";
	    }
	} else {
	    print LOG "didn't chown/chgrp/chmod $mycdir\n";
	}
    } else {
	print LOG "didn't chown/chgrp/chmod cgi scripts or $mycdir\n";
    }
    print "\nSet owner/group/permissions for $myrdir [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	$fail = "no";
	print "File owner: [$user] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $user = $ans;
	}
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	print "Permissions: [775] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $perm = $ans;
	} else {
	    $perm = "775";
	}
	$ret = system("chown $user $myrdir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chown $user $myrdir failed!\n";
	}
	$ret = system("chgrp $group $myrdir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chgrp $group $myrdir failed!\n";
	}
	$ret = system("chmod $perm $myrdir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chmod $perm $myrdir failed!\n";
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for $myrdir \n";
	} else {
	    print LOG "chown/chgrp/chmod failed for $myrdir \n";
	}
    } else {
	print LOG "didn't chown/chgrp/chmod $myrdir\n";
    }
    print "\nSet owner/group/permissions for $myadir [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	$fail = "no";
	print "File owner: [$user] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $user = $ans;
	}
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	print "Permissions: [775] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $perm = $ans;
	} else {
	    $perm = "775";
	}
	$ret = system("chown $user $myadir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chown $user $myadir failed!\n";
	}
	$ret = system("chgrp $group $myadir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chgrp $group $myadir failed!\n";
	}
	$ret = system("chmod $perm $myadir");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chmod $perm $myadir failed!\n";
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for $myadir\n";
	} else {
	    print LOG "chown/chgrp/chmod failed for $myadir \n";
	}
    } else {
	print LOG "didn't chown/chgrp/chmod $myrdir\n";
    }
    # setuid
    print "\nThe routewatch program must run as setuid to root.\n";
    print " (routewatch accesses an IGMP socket, hence root) \n";
    $dosetuid = "yes";
    print "make routewatch setuid to root [yes] ";
    $yes = <STDIN>;
    if (($yes eq "\n") || ($yes =~ "y") || ($yes =~ "Y")) {
	print "File group: [$group] ";
	$ans = <STDIN>;
	if ($ans ne "\n") {
	    chop($ans);
	    $group = $ans;
	}	
	$ret = system("chown root $mybin/routewatch");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chown root $mybin/routewatch failed!\n";
	}
	$ret = system("chgrp $group $mybin/routewatch ");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chgrp $group $mybin/routewatch failed!\n";
	}
	$ret = system("chmod 755 $mybin/routewatch");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chmod 755 $mybin/routewatch failed!\n";
	}
	$ret = system("chmod u+s $mybin/routewatch");
	if ($ret != 0) {
	    $fail = "yes";
	    print "chmod u+s $mybin/routewatch failed!\n";
	}
	if ($fail eq "no") {
	    print LOG "chown/chgrp/chmod completed for $mybin/routewatch \n";
	    $dosetuid = "no";
	} else {
	    print LOG "chown/chgrp/chmod failed for $mybin/routewatch \n";
	}
    }
    if ($dosetuid ne "no") {
	print "\nAfter the install has completed, the file:\n";
	print "$mybin/routewatch\n";
	print "must be set so routemonitor can run it as setuid to root.\n";
    }
}
    

sub messages {
    print "Installation completed.\n\n\n";
    if (($webserver eq "no") || ($cgiserver eq "no") || ($dosetuid eq "yes")) {
	print "There are some additional installation steps that could\n";
	print "not be performed by this script.  You will need to \n";
	print "complete these steps before running the RouteMonitor.\n";
	print "See the file ./install.additional for detailed instructions\n";
	print  "\n";
	open(ADD,"> ./install.additional") || 
	    die "unable to create ./install.additional\n";
	if ($webserver eq "no") {
	    print ADD "Create the script $mycopy\n";
	    print ADD "  This script should copy all files from $mywdir\n";
	    print ADD "  onto a directory on the webserver.\n";
	}
	if ($cgiserver eq "no") {
	    print ADD "Copy the files in $mycdir \n";
	    print ADD "  into a CGI directory on the web server.\n";
	    print ADD "  URL's for this CGI directory should begin with:\n";
	    print ADD "  $myhref\n";
	}
	if ($dosetuid eq "yes") {
	    print ADD "The file $mybin/routewatch must be setuid to root\n";
	    print ADD "  set the owner of the file to root.\n";
	    print ADD "  change the permissions so it setuid\n";
	    print ADD "  make sure the routemonitor program has permission \n";
	    print ADD "  to run it.\n";
	    print ADD "  chown root $mybin/routewatch\n";
	    print ADD "  chgrp $group $mybin/routewatch\n";
	    print ADD "  chmod 755 $mybin/routewatch\n";
	    print ADD "  chmod u+s $mybin/routewatch\n";
	}
	close(ADD);
    }
    print "A installation summary is contained in ./install.log\n";
    print "\n\nA script has been provided to start/stop the RouteMonitor\n";
    print "package.  (This script is also useful for enabling RouteMonitor\n";
    print "to start automatically whenever the machine is booted).\n";
    print "\n";
    print "You may start (stop) RouteMontior with the command: \n";
    print "$mybin/RouteMonitor start (stop)\n";
    print "You may also start RouteMonitor by directly with the command:\n";
    print "$mybin/htmlmaker $opts\n";
}

sub get_topology {
    print "\n";
    print "The data displayed on your website will be more meaningful\n";
    print "if the viewer has some sense of where the data is collected.\n";
    print "A RouteMonitor website includes about.html file which \n";
    print "describes how RouteMonitor operates and should include a brief \n";
    print "paragraph describing the topology where this data is collected\n";
    print "\n";
    print "For example, the UCLA RouteMonitor website describes it's\n";
    print "topology as:\n";
    print "  UCLA has a router with a tunnel to the MBone and several\n";
    print "  local routers which distribute mcast to local subnets.\n";
    print "  This data is collected from the subnet where the main\n";
    print "  router exchanges updates with the local routers.\n";
    print "\n";
    print "Please enter a brief description of your RouteMonitor location.\n";
    print "When finished, enter control-D on a new line.\n";
    @mytopo = <STDIN>;
    print "\n";
    print "If you later wish to change this information, edit the file\n"; 
    print "$mywdir/about.html\n";
    print "\n";
}

sub register {
    open(ADD,">> ./install.additional") || 
	die "unable to create ./install.additional\n";
    print "\n";
    print ADD "\n";
    print ADD "There is a RouteMonitor Home page with links to multicast\n";
    print ADD "RouteMonitors running at various sites.  The home page is\n";
    print ADD "located at http://ganef.cs.ucla.edu/~masseyd/Route.\n";
    print ADD "Please add your site to this list by sending your URL to: \n";
    print ADD 'masseyd@cs.ucla.edu'; print "\n";
    close(ADD);
    print "\n";
    print "There is a RouteMonitor Home page with links to multicast\n";
    print "RouteMonitors running at various sites.  The home page is\n";
    print "located at http://ganef.cs.ucla.edu/~masseyd/Route.\n";
    print "Please add your site to this list by sending your URL to: \n";
    print 'masseyd@cs.ucla.edu';  print "\n";
    print LOG 'Please send your URL to masseyd@cs.ucla.edu'; print "\n";
}


